//-----------------------------------------------------------------------------
// RECIRCULATING MODEL FOR O2-CO2 TRANSPORT AND EXCHANGE BETWEEN LUNG AND A
// SYSTEMIC ORGAN (SAY HEART). INSTANTANEOUS EQUILIBRIUM BINDING OF O2 AND 
// CO2 WITH HEMOGLOBIN AND MYOGLOBIN IS CONSIDERED. THE SUBSCRIPTS BELOW "11", 
// "21" AND "22" INDICATE "PULMONARY CAPILLARY", "SYSTEMIC CAPILLARY" AND 
// "SYSTEMIC ISF" RESPECTIVELY.
//
// LAST MODIFIED ON 2/25/2002
//-----------------------------------------------------------------------------
     JSim v1.1

     import nsrunit;
     unit conversion on;

     math o2co2recirc {

     realDomain t sec;	// TIME DOMAIN
     t.min=0; t.max=60; t.delta=0.05;
//-----------------------------------------------------------------------------
// THE TRANSPORT AND EXCHANGE PARAMETERS. ASSUMING THAT THE ORGAN IS HEART
// AND ITS WEIGHT IS 5 KG. THE LUNG WEIGHT IS ALSO CONSIDERED AS 5 KG.
   real
     V11 = 0.075 L,		// PULM CAP volume
     V21 = 0.350 L,		// SYST CAP volume
     V22 = 4 L,			// SYST ISF volume
     F = 5 L/min,		// blood flow rate (cardiac output)
     PSO1 = 1320 L/min,		// PS for O2 across PULM CAP membrane
     PSO2 = 1000 L/min,		// PS for O2 across SYST CAP membrane
     PSC1 = 1155 L/min,		// PS for CO2 across PULM CAP membrane
     PSC2 = 0875 L/min,		// PS for CO2 across SYST CAP membrane
     PSB2 = 11.55 L/min,	// PS for HCO3- across SYST CAP memrane
     PSH2 = 115.5 L/min,	// PS for H+ across SYST CAP memrane
     GDR2 = 0.63,		// Gibbs Donnan ratio across SYST CAP
     CFbl = 6000,		// catalytic factor in blood due to CA
     CFti = 10000,		// catalytic factor in tissue due to CA
     BETAbl = 30e-3 M,		// buffering capacity in blood
     BETAti = 36e-3 M;		// buffering capacity in tissue
//-----------------------------------------------------------------------------
// THE PARAMETERS FOR O2 AND CO2 REACTIONS WITH HEMOGLOBIN AND MYOGLOBIN.
// O2capHb = (4 moles of O2/mole of Hb)*(22400 ml of O2/mole of O2)*
//           (1 mole of Hb/65000 gm of Hb) = 1.38 ml of O2/gm of Hb.
// CO2capHb = O2capHb/4 = 0.345 ml of CO2/gm of Hb.
// O2concHb = 4*CHb = (O2capHb/mol2ml)*(AmtHb*Hct/0.45) in M.
// CO2concHb = O2concHb/4 = CHb = (CO2capHb/mol2ml)*(AmtHb*Hct/0.45) in M.
//------------------------------------------------------------------------------
   real
     alphaO2 = 1.35e-6 M/mmHg,	// O2 solubility coefficient
     alphaCO2 = 3.0e-5 M/mmHg,	// CO2 solubility coefficient
     kp1 = 0.12 1/sec,		// forward rate constant in CO2+H2O reaction
     km1 = 16.2e4 1/M/sec,	// backward rate constant in CO2+H2O reaction
     K1 M,			// EQUIL constant in CO2+H2O reaction (kp1/km1)
     K2 = 1.38e-4,		// EQUIL constant in CO2+HbNH2 reaction
     K3 = 3.45e-5,		// EQUIL constant in CO2+O2HbNH2 reaction   
     K4 = 6.9e-8 M,		// EQUIL constant in HbNH3+ ionization
     K5 = 1.38e-7 M,		// EQUIL constant in O2HbNH3+ ionization   
     K6p = 4.96e4 1/M^1.636,	// EQUIL constant in O2+HbNH2 reaction
     mol2ml = 22400 ml/mol,	// conversion from mole to ml (for gas)
     P50Mb = 2.4 mmHg,		// P50 for oxygen saturation of myoglobin
     CMb = 0.5e-3 M,		// concentration of myoglobin in tissue
     Hct = 0.45,		// hematocrit (volume fraction of RBCs in blood)
     AmtHb = 150 g/L,		// ammount of Hb in gm per liter of blood
     MWHb = 65000 g/mol,	// molecular weight of hemoglobin
     CHb M,			// concentration of hemoglobin in blood
     O2concHb M,		// maximum O2 concentration in bound form with Hb
     CO2concHb M,		// maximum CO2 concentration in bound form with Hb 
     O2concMb M;		// maximum O2 concentration in bound form with Mb

     K1 = kp1/km1;
     CHb = (AmtHb/MWHb)*(Hct/0.45);
     O2concHb = 4*CHb;
     CO2concHb = CHb;
     O2concMb = CMb;
//------------------------------------------------------------------------------
// DEPENDENT VARIABLES WHICH ARE COMPUTED AS A PART OF SOLUTIONS.
   real
     PO11(t) mmHg,		// O2 partial pressure in PULM CAP
     PO21(t) mmHg,		// O2 partial pressure in SYST CAP
     PO22(t) mmHg,		// O2 partial pressure in SYST ISF
     O11(t) M,			// free O2 concentration in PULM CAP 
     O21(t) M, 			// free O2 concentration in SYST CAP
     O22(t) M, 			// free O2 concentration in SYST ISF
     TO11(t) M, 		// total O2 concentration in PULM CAP
     TO21(t) M,			// total O2 concentration in SYST CAP 
     TO22(t) M,			// total O2 concentration in SYST ISF
     CHbO11(t) M, 		// Hb-bound O2 concentration in PULM CAP
     CHbO21(t) M,		// Hb-bound O2 concentration in SYST CAP
     CMbO22(t) M,		// Mb-bound O2 concentration in SYST ISF
     KO11(t) M^(-2.65),		// Hills coefficient for HbO2 in PULM CAP 
     KO21(t) M^(-2.65),		// Hills coefficient for HbO2 in SYST CAP
     KO22(t) M^(-1),		// Hills coefficient for MbO2 in SYST ISF
     SHbO11(t),			// HbO2 saturation in PULM CAP (unitless)
     SHbO21(t),			// HbO2 saturation in SYST CAP (unitless)
     SMbO22(t), 		// MbO2 saturation in SYST ISF (unitless)
     DSHbO11(t) M^(-1),		// change in SHbO11 w.r.t. O11 (dSHbO11/dO11) 
     DSHbO21(t) M^(-1),		// change in SHbO21 w.r.t. O21 (dSHbO21/dO21)
     DSMbO22(t) M^(-1),		// change in SMbO22 w.r.t. O22 (dSMbO22/dO22)
     CRO11(t),			// CONC ratio for O2 in PULM CAP (TO11/O11)
     CRO21(t),			// CONC ratio for O2 in SYST CAP (TO21/O21)
     CRO22(t),			// CONC ratio for O2 in SYST ISF (TO22/O22)
     VRO11(t),			// VOL ratio for O2 in PULM CAP (dTO11/dO11)
     VRO21(t),			// VOL ratio for O2 in SYST CAP (dTO21/dO21)
     VRO22(t),			// VOL ratio for O2 in SYST ISF (dTO22/dO22)
     VVO11(t) L,		// virtual VOL for O2 in PULM CAP (V11*VRO11)
     VVO21(t) L,		// virtual VOL for O2 in SYST CAP (V21*VRO21)
     VVO22(t) L,		// virtual VOL for O2 in SYST ISF (V22*VRO22)

     PC11(t) mmHg,		// CO2 partial pressure in PULM CAP
     PC21(t) mmHg,		// CO2 partial pressure in SYST CAP
     PC22(t) mmHg,		// CO2 partial pressure in SYST ISF
     C11(t) M,			// free CO2 concentration in PULM CAP 
     C21(t) M,			// free CO2 concentration in PULM CAP 
     C22(t) M,			// free CO2 concentration in SYST CAP
     TC11(t) M,			// total CO2 concentration in PULM CAP
     TC21(t) M,			// total CO2 concentration in SYST CAP 
     CHbC11(t) M,		// Hb-bound CO2 concentration in PULM CAP
     CHbC21(t) M, 		// Hb-bound CO2 concentration in SYST CAP
     KC11(t) M^(-1),		// Hills coefficient for HbCO2 in PULM CAP
     KC21(t) M^(-1),		// Hills coefficient for HbCO2 in SYST CAP
     SHbC11(t), 		// HbCO2 saturation in PULM CAP (unitless)
     SHbC21(t),			// HbCO2 saturation in SYST CAP (unitless)
     DSHbC11(t) M^(-1),		// change in SHbC11 w.r.t. C11 (dSHbC11/dC11) 
     DSHbC21(t) M^(-1),		// change in SHbC21 w.r.t. C21 (dSHbC21/dC21)
     CRC11(t),			// CONC ratio for C2 in PULM CAP (TC11/C11)
     CRC21(t),			// CONC ratio for C2 in SYST CAP (TC21/C21)
     VRC11(t),			// VOL ratio for C2 in PULM CAP (dTC11/dC11)
     VRC21(t),			// VOL ratio for C2 in SYST CAP (dTC21/dC21)
     VVC11(t) L,		// virtual VOL for C2 in PULM CAP (V11*VRC11)
     VVC21(t) L,		// virtual VOL for C2 in SYST CAP (V21*VRC21)

     B11(t) M,			// concentration of HCO3- ions in PULM CAP
     B21(t) M,			// concentration of HCO3- ions in SYST CAP
     B22(t) M,			// concentration of HCO3- ions in SYST ISF
     H11(t) M,			// concentration of H+ ions in PULM CAP
     H21(t) M,			// concentration of H+ ions in SYST CAP
     H22(t) M,			// concentration of H+ ions in SYST ISF
     pH11(t),			// pH in PULM CAP (unitless)
     pH21(t),			// pH in SYST CAP (unitless)
     pH22(t);			// pH in SYST ISF (unitless)
//-----------------------------------------------------------------------------
// INITIAL CONDITIONS. INITIAL CONCENTRATIONS OF H+ AND HCO3- IONS ARE 
// OBTAINED ASSUMING THAT THE INITIAL PO2 AND PCO2 ARE KNOWN.
   real
     Hnorm = 1 M,		// H+ ion normalization variable
     pH110 = 7.30,		// pH in PULM CAP at t=0
     pH210 = 7.40,		// pH in SYST CAP at t=0
     pH220 = pH210 + log(GDR2);	// pH in SYST ISF at t=0

   when (t=t.min) {
     PO11 = 040;
     PO21 = 100;
     PO22 = 025;
     PC11 = 46;
     PC21 = 40;
     PC22 = 65;
     H11 = Hnorm*10^(-pH110);
     H21 = Hnorm*10^(-pH210);	
     H22 = Hnorm*10^(-pH220);
     B11 = K1*alphaCO2*PC11/H11;
     B21 = K1*alphaCO2*PC21/H21;
     B22 = K1*alphaCO2*PC22/H22;  }
// ----------------------------------------------------------------------------
// EQUATIONS FOR PARTIAL PRESSURE OF O2 AND CO2 IN ALVEOLI (UNDER CONTINUOUS 
// BREATHING CONDITION). THE SUBSCRIPT "12" DENOTES THE ALVEOLAR REGION.
   real
     Pbaro = 760 mmHg,		// barometric pressure of the air
     PO2insp = 150 mmHg,	// partial pressure of O2 in the inspired air
     PH2O = 47 mmHg,		// partial pressure of H2O in the inspired air
     RQ = 0.8,			// respiratory quotient (unitless)
     VdotO2 = 250 ml/min,	// O2 uptake rate in lungs (250 ml/min)
     VdotCO2 = RQ*VdotO2,	// CO2 removal rate in lungs (200 ml/min)
     Vdotalv = 3565 ml/min,	// alveolar ventilation
     PO12 mmHg,			// partial pressure of O2 in alveoli
     PC12 mmHg,			// partial pressure of CO2 in alveoli
     O12 M,			// concentration of O2 in alveoli
     C12 M;			// concentration of CO2 in alveoli

     PO12 = PO2insp - (VdotO2/Vdotalv)*(Pbaro-PH2O);
     PC12 = (VdotCO2/Vdotalv)*(Pbaro-PH2O);
     O12 = alphaO2*PO12;
     C12 = alphaCO2*PC12;
// ----------------------------------------------------------------------------
// EQUATIONS for O2 CONSUMPTION RATE AND CO2 PRODUCTION RATE IN TISSUE.
   real
     Kcytox = 7e-7 M,		// Kmax for cytocrome oxydase in SYST ISF
     Vcytox mol/min,		// Vmax for cytocrome oxydase in SYST ISF
     GO22(t) ml/min,		// gulosity for O2 consumption in SYST ISF
     MRO22(t) mol/min,		// regional O2 consumption rate in SYST ISF
     MRC22(t) mol/min;		// regional CO2 production rate in SYST ISF

     Vcytox = VdotO2/mol2ml;
     GO22 = Vcytox/(Kcytox+O22);
     MRO22 = GO22*O22;
     MRC22 = RQ*MRO22;
//-----------------------------------------------------------------------------
// CONVERSION FROM PARTIAL PRESSURE TO CONCENTRATION. CONVERSION FROM
// HYDROGEN ION CONCENTRATION TO pH.
     O11 = PO11*alphaO2;
     O21 = PO21*alphaO2;
     O22 = PO22*alphaO2;
     C11 = PC11*alphaCO2;
     C21 = PC21*alphaCO2;
     C22 = PC22*alphaCO2;
     pH11 = -log(H11/Hnorm);
     pH21 = -log(H21/Hnorm);
     pH22 = -log(H22/Hnorm);
// ----------------------------------------------------------------------------
// EQUATIONS FOR HILLS COEFFICIENTS (KO11, KO21, KO22, KC11, AND KC22), 
// SATURATIONS (SHbO11, SHbO21, SMbO22, SHbC11, AND SHbC21), AND MANY
// OTHER RELATED VARIABLES
     KO11 = (1 + (K3/H11)*C11 + (H11/K5))*(K6p/H11^1.014)/
            (1 + (K2/H11)*C11 + (H11/K4));
     SHbO11 = KO11*O11^2.65/(1+KO11*O11^2.65);
     CHbO11 = O2concHb*SHbO11;
     TO11 = O11 + CHbO11;
     CRO11 = if (O11=0) 1 else TO11/O11;
     DSHbO11 = 2.65*KO11*O11^1.65/(1+KO11*O11^2.65)^2;
     VRO11 = 1 + O2concHb*DSHbO11;
     VVO11 = V11*VRO11;

     KO21 = (1 + (K3/H21)*C21 + (H21/K5))*(K6p/H21^1.014)/
            (1 + (K2/H21)*C21 + (H21/K4));
     SHbO21 = KO21*O21^2.65/(1+KO21*O21^2.65);
     CHbO21 = O2concHb*SHbO21;
     TO21 = O21 + CHbO21;
     CRO21 = if (O21=0) 1 else TO21/O21;
     DSHbO21 = 2.65*KO21*O21^1.65/(1+KO21*O21^2.65)^2;
     VRO21 = 1 + O2concHb*DSHbO21;
     VVO21 = V21*VRO21;

     KO22 = 1/(alphaO2*P50Mb);
     SMbO22 = KO22*O22/(1+KO22*O22);
     CMbO22 = O2concMb*SMbO22;
     TO22 = O22 + CMbO22;
     CRO22 = if (O22=0) 1 else TO22/O22;
     DSMbO22 = KO22/(1+KO22*O22)^2;
     VRO22 = 1 + O2concMb*DSMbO22;
     VVO22 = V22*VRO22;

     KC11 = (K2 + K3*(K6p/H11^1.014)*O11^2.65)/(((1+H11/K4)+
            (1+H11/K5)*(K6p/H11^1.014)*O11^2.65)*H11);
     SHbC11 = KC11*C11/(1+KC11*C11);
     CHbC11 = CO2concHb*SHbC11;
     TC11 = C11 + CHbC11;
     CRC11 = if (C11=0) 1 else TC11/C11;
     DSHbC11 = KC11/(1+KC11*C11)^2;
     VRC11 = 1 + CO2concHb*DSHbC11;
     VVC11 = V11*VRC11;

     KC21 = (K2 + K3*(K6p/H21^1.014)*O21^2.65)/(((1+H21/K4)+
            (1+H21/K5)*(K6p/H21^1.014)*O21^2.65)*H21);
     SHbC21 = KC21*C21/(1+KC21*C21);
     CHbC21 = CO2concHb*SHbC21;
     TC21 = C21 + CHbC21;
     CRC21 = if (C21=0) 1 else TC21/C21;
     DSHbC21 = KC21/(1+KC21*C21)^2;
     VRC21 = 1 + CO2concHb*DSHbC21;
     VVC21 = V21*VRC21;
//-----------------------------------------------------------------------------
// ODEs FOR OXYGEN, CARBON DIOXIDE, BICARBONATE IONS, AND HYDROGEN IONS IN 
// PULMONARY CAPILLARY, SYSTEMIC CAPILLARY, AND SYSTEMIC ISF REGIONS.
     PO11:t = (F*(CRO21*PO21-CRO11*PO11) + PSO1*(PO12-PO11))/VVO11;
     PO21:t = (F*(CRO11*PO11-CRO21*PO21) - PSO2*(PO21-PO22))/VVO21;
     PO22:t = (PSO2*(PO21-PO22) - MRO22/alphaO2)/VVO22;

     PC11:t = (F*(CRC21*PC21-CRC11*PC11) - PSC1*(PC11-PC12) -
              V11*CFbl*(kp1*PC11-(km1/alphaCO2)*B11*H11))/VVC11;
     PC21:t = (F*(CRC11*PC11-CRC21*PC21) + PSC2*(PC22-PC21) - 
              V21*CFbl*(kp1*PC21-(km1/alphaCO2)*B21*H21))/VVC21;
     PC22:t = (MRC22/alphaCO2 - PSC2*(PC22-PC21))/V22 -
              CFti*(kp1*PC22-(km1/alphaCO2)*B22*H22);

     B11:t = (F/V11)*(B21-B11) + CFbl*(kp1*C11-km1*B11*H11);
     B21:t = (F/V21)*(B11-B21) - (PSB2/V21)*(GDR2*B21-B22) + 
             CFbl*(kp1*C21-km1*B21*H21);
     B22:t = (PSB2/V22)*(GDR2*B21-B22) + CFti*(kp1*C22-km1*B22*H22);

     H11:t = (F/V11)*(H21-H11) +
             CFbl*(2.303/BETAbl)*H11*(kp1*C11-km1*H11*B11);
     H21:t = (F/V21)*(H11-H21) - (PSH2/V21)*(H21-GDR2*H22) + 
             CFbl*(2.303/BETAbl)*H21*(kp1*C21-km1*H21*B21);
     H22:t = (PSH2/V22)*(H21-GDR2*H22) + 
             CFti*(2.303/BETAti)*H22*(kp1*C22-km1*H22*B22);
}
//-----------------------------------------------------------------------------
